//
//  main.c
//  Created by Leotescu Vladut.
//

// AVR headers
#include <avr/io.h>
#include <avr/interrupt.h>
#define F_CPU 16000000
#include <util/delay.h>

// User defined headers 
#include "timers.h"
#include "utils.h"

// Define virtual timers
void timer10ms();
void timer500ms();
const unsigned int tmr_count = 2;
const unsigned int tmr_max_value = 99;
const unsigned int tmr_call[] = {1, 50}; // call * 10ms
const void_func_ptr tmr_func_ptr[] = {timer10ms, timer500ms};

// Define piano syllables for PWM frequency
const uint8_t note_freq[8] = {
	// DO    RE    MI    FA    SOL   LA    SI    DO
	   48,   60,   72,   84,   96,  108,  120,  132
};

// Define piano possible states (NORMAL, RECORDING OR PLAYBACK)
#define NORMAL		0
#define RECORDING	1
#define PLAYBACK	2

// Define piano current state and state timeout
uint8_t piano_state;
uint8_t piano_state_timeout;

// Define last recorded song
song_note last_song[MAX_SONG_SIZE];
unsigned int rec_pos;
unsigned int play_pos;
uint8_t last_note_freq;
uint16_t last_note_duration;

int main(void) {

	// Init timers
	init_timers();

	// Set the output pins
	DDRA |= _BV(GREEN_LED) | _BV(RED_LED) | _BV(YELLOW_LED);	// Yellow , Green and red LED
	DDRD = _BV(PD5);						// Speaker

	// Set the input pins
	DDRC = 0;								// Keyboard push buttons
	PORTC = 0xff;							// Keyboard pull-up resistances
	DDRC &= (~_BV(0) & ~_BV(6)) & (~_BV(1) & ~_BV(6)) &
		 (~_BV(1) & ~_BV(6));		// Normal, Record and Play push buttons
	PORTC |= (_BV(0) & _BV(6)) | (_BV(1) & _BV(6)) | (_BV(2) & _BV(6)); // Pull-up resistances

	// Set to default piano state - NORMAL
	led_on(&PORTA, RED_LED);
	piano_state = NORMAL;
	piano_state_timeout = 0;

	// Wait for interrupts
	while(1) {;}

	return 0;
}

// This functions is called every 10ms using Timer0 interrupt
void timer10ms(void) {
	int i, rc;
	int play_freq = 0;
	int tot_pushed_btns = 0;

	// Check for pushet control buttons
	// Record button
	if (get_key_numeric() == 10 && piano_state_timeout == 0) {
		switch (piano_state) {
		case NORMAL:
			piano_state = RECORDING;
			led_off(&PORTA, RED_LED);
			led_on(&PORTA, GREEN_LED);

			// Init data for song recording
			init_recording();

			// Set timeout
			piano_state_timeout = STATE_TIMEOUT;
			break;
		case RECORDING:
			piano_state = NORMAL;
			led_off(&PORTA, GREEN_LED);
			led_on(&PORTA, RED_LED);
			piano_state_timeout = STATE_TIMEOUT;
			break;
		case PLAYBACK:
		default:
			break;
		}
	}

	// Playback button
	if (get_key_numeric() == 11 && piano_state_timeout == 0) {
		switch (piano_state) {
		case NORMAL:
			piano_state = PLAYBACK;
			led_off(&PORTA, RED_LED);
			led_on(&PORTA, YELLOW_LED);
			// Init data for song playback
			init_playback();

			// Set timeout
			piano_state_timeout = STATE_TIMEOUT;
			break;
		case PLAYBACK:
			piano_state = NORMAL;
			led_off(&PORTA, YELLOW_LED);
			led_on(&PORTA, RED_LED);
			piano_state_timeout = STATE_TIMEOUT;
			break;
		case RECORDING:
		default:
			break;
		}
	}

	// Actions in NORMAL and RECORDING state
	if (piano_state == NORMAL || piano_state == RECORDING) {

		// Check for pushed piano buttons. If more than one piano buttons are
		// pushed, calculate the average frequency
		for (i = 0; i < 8; i++) {
			if (get_key_numeric() == i && piano_state_timeout == 0)
				play_freq += note_freq[i];
				tot_pushed_btns++;
			}
		}
					

		// Average frequency
		if(tot_pushed_btns > 0)
				play_freq = play_freq / tot_pushed_btns;
		else
				play_freq = 0;

		if (piano_state == RECORDING) {
			rc = record_song(play_freq);

			// Max song size reached
			if (rc == -1) {
				piano_state = NORMAL;
				led_off(&PORTA, GREEN_LED);
				led_on(&PORTA, RED_LED);
			}
		}
	

	// Actions in PLAYBACK state
	if (piano_state == PLAYBACK) {
		rc = playback_song(&play_freq);

		// Song ended
		if (rc == -1) {
			piano_state = NORMAL;
			led_on(&PORTA, RED_LED);
		}
	}

	// Play syllable
	if (play_freq > 0) {
		set_pwm_freq(play_freq);
		pwm_on();
	} else {
		pwm_off();
	}

	// Decrease piano state timeout (state is changed only when timeout is 0)
	piano_state_timeout = (piano_state_timeout > 0)? piano_state_timeout - 1: 0;
	if(piano_state_timeout > 0)
		piano_state_timeout--; 
	else 
		piano_state_timeout = 0;
}

// This functions is called every 500ms using Timer0 interrupt
void timer500ms(void) {

	if (piano_state == RECORDING) {
		led_toggle(&PORTA, GREEN_LED);
	}

	if (piano_state == PLAYBACK) {
		led_toggle(&PORTA, YELLOW_LED);
	}

	if (piano_state == NORMAL) {
		led_toggle(&PORTA, RED_LED);
	}
}